<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Filesystem\Stack;

// Import framework stuff
use Org\Mxchange\CoreFramework\Filesystem\Block;
use Org\Mxchange\CoreFramework\Filesystem\File\BaseBinaryFile;
use Org\Mxchange\CoreFramework\Generic\UnsupportedOperationException;

// Import SPL stuff
use \SplFileInfo;

/**
 * A stack file class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class StackFile extends BaseBinaryFile implements Block {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this File class and prepares it for usage
	 *
	 * @param	$infoInstance	An instance of a SplFileInfo class
	 * @param	$blockInstance	An instance of a Block class
	 * @return	$fileInstance	An instance of this File class
	 */
	public final static function createStackFile (SplFileInfo $infoInstance, Block $blockInstance) {
		// Get a new instance
		$fileInstance = new StackFile();

		// Set block instance here for callbacks
		$fileInstance->setBlockInstance($blockInstance);

		// Init this abstract file
		$fileInstance->initFile($infoInstance);

		// Return the prepared instance
		return $fileInstance;
	}

	/**
	 * Writes given value to the file and returns a hash and gap position for it
	 *
	 * @param	$groupId	Group identifier
	 * @param	$value		Value to be added to the stack
	 * @return	$data		Hash and gap position
	 */
	public function writeValueToFile ($groupId, $value) {
		// Make sure no objects/resources are added as the serialization may fail
		assert(!is_object($value));
		assert(!is_resource($value));

		// Encode/convert the value into a "binary format"
		$encoded = $this->encodeData($value);

		// Get a strong hash for the "encoded" data
		$hash = self::hash($encoded);

		// Then write it to the next free gap
		$data = $this->getBlockInstance()->writeDataToFreeGap($groupId, $hash, $encoded);

		// Return info
		return $data;
	}

	/**
	 * Writes given raw data to the file and returns a gap position and length
	 *
	 * @param	$groupId	Group identifier
	 * @param	$hash		Hash from encoded value
	 * @param	$encoded	Encoded value to be written to the file
	 * @return	$data		Gap position and length of the raw data
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function writeDataToFreeGap ($groupId, $hash, $encoded) {
		self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('[' . __METHOD__ . ':' . __LINE__ . '] groupId=' . $groupId . ',hash=' . $hash . ',encoded()=' . strlen($encoded));
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

}
